@@ -1,3 +1,26 @@
+# This file is part of OSMSG (https://github.com/kshitijrajsharma/OSMSG).
+# MIT License
+
+# Copyright (c) 2023 Kshitij Raj Sharma
+
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+
+# The above copyright notice and this permission notice shall be included in all
+# copies or substantial portions of the Software.
+
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
+
 import argparse
 import concurrent.futures
 import datetime as dt
@@ -32,15 +55,17 @@ from .changefiles import (
     strip_utc,
 )
 from .changesets import ChangesetToolKit
+from .login import verify_me_osm
 from .utils import (
     create_charts,
     create_profile_link,
     download_osm_files,
+    extract_projects,
+    generate_tm_stats,
     get_file_path_from_url,
     sum_tags,
     update_stats,
     update_summary,
-    verify_me_osm,
 )
 
 users_temp = {}
@@ -490,6 +515,13 @@ def parse_args():
         default=False,
     )
 
+    parser.add_argument(
+        "--tm_stats",
+        action="store_true",
+        help="Includes Tasking Manager stats for users , TM Projects are filtered from hashtags used , Appends all time stats for user for project id produced from stats",
+        default=False,
+    )
+
     parser.add_argument(
         "--rows",
         type=int,
@@ -650,11 +682,17 @@ def main():
             )
             sys.exit()
         old_csv_path = os.path.join(os.getcwd(), f"{fname}.csv")
-        if not os.path.exists(old_csv_path):
-            print(f"Error: Couldn't find old stats csv at :{old_csv_path}")
-        old_df = pd.read_csv(old_csv_path, encoding="utf8")
-        args.start_date = str(old_df.iloc[0]["end_date"])
-        old_stats_start_date = str(old_df.iloc[0]["start_date"])
+        old_summary_path = os.path.join(os.getcwd(), f"{fname}_summary.csv")
+        if not os.path.exists(old_csv_path) or not os.path.exists(old_summary_path):
+            print(
+                f"Error: Couldn't find old stats/summary csv at :{old_csv_path} hence changing update to false and extracting last day stats for default"
+            )
+            args.update = False
+            args.last_day = True
+        if args.update:
+            old_df = pd.read_csv(old_csv_path, encoding="utf8")
+            args.start_date = str(old_df.iloc[0]["end_date"])
+            old_stats_start_date = str(old_df.iloc[0]["start_date"])
 
     if args.start_date:
         start_date = strip_utc(
@@ -696,6 +734,11 @@ def main():
             )
         print(f"Ignoring --url , and using Geofabrik Update URL for {args.country}")
         args.url = osc_url_temp
+
+    if args.tm_stats:
+        if not args.changeset and not args.hashtags:
+            args.changeset = True  # changeset is required to extract tm project id from hashtags field
+
     if args.changeset:
         if args.hashtags:
             assert (
@@ -995,6 +1038,7 @@ def main():
         df = df.drop(columns=["changes"])
         df = df.sort_values("map_changes", ascending=False)
         df.insert(0, "rank", range(1, len(df) + 1), True)
+
         if args.rows:
             df = df.head(args.rows)
         print(df)
@@ -1028,6 +1072,45 @@ def main():
                         )
                     )
 
+        if args.tm_stats:
+            print("Generating TM Stats ....")
+            # apply function to create new column with list of projects
+            df1 = df
+            df1["tm_projects"] = df1["hashtags"].apply(extract_projects)
+            unique_projects = list(
+                set(
+                    project
+                    for project_list in df1["tm_projects"]
+                    for project in project_list
+                )
+            )
+            usernames_unique = df1["name"].unique().tolist()
+
+            df2 = generate_tm_stats(unique_projects, usernames_unique)
+            # explode projects column to create new row for each project
+            df1 = df1.explode("tm_projects")
+
+            # merge df1 and df2_grouped on username and project
+            df_merged = pd.merge(df1, df2, on=["name", "tm_projects"], how="left")
+
+            # group df_merged by username to combine information for each unique username
+            df_final = (
+                df_merged.groupby(["name"])
+                .agg(
+                    {
+                        "tm_mapping_level": "first",
+                        "tasks_mapped": "sum",
+                        "tasks_validated": "sum",
+                        "tasks_total": "sum",
+                    }
+                )
+                .reset_index()
+            )
+
+            df = pd.merge(df, df_final, on=["name"], how="left")
+            df["tm_projects"] = df["tm_projects"].apply(lambda x: ",".join(x))
+            print(df)
+
         if args.summary:
             summary_df = pd.json_normalize(list(summary_interval.values()))
             summary_df = summary_df.assign(
@@ -1046,11 +1129,6 @@ def main():
 
             summary_df = summary_df.sort_values("timestamp", ascending=True)
             if args.update:
-                old_summary_path = os.path.join(os.getcwd(), f"{fname}_summary.csv")
-                if not os.path.exists(old_summary_path):
-                    print(
-                        f"Error: Couldn't find old summary csv at :{old_summary_path}"
-                    )
                 old_summary = pd.read_csv(f"{fname}_summary.csv", encoding="utf8")
                 summary_df = update_summary(old_summary, summary_df)
 
@@ -1191,6 +1269,9 @@ def main():
                 if args.tags:
                     for user_tag in args.tags:
                         user_tags_summary += f"- {user_tag} = Created: {humanize.intword(df[f'{user_tag}.create'].sum())}, Modified : {humanize.intword(df[f'{user_tag}.modify'].sum())}\n"
+                if args.length:
+                    for len_feat in args.length:
+                        user_tags_summary += f"- {len_feat} length created = {humanize.intword(round(df[f'{len_feat}_len_m'].sum()/1000))} Km\n"
                 file.write(f"{user_tags_summary}\n")
 
                 if args.all_tags:
