@@ -1,25 +1,25 @@
-#! /usr/bin/env python3
-# Copyright 2018 Geofabrik GmbH
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are met:
+# This file is part of OSMSG (https://github.com/kshitijrajsharma/OSMSG).
+# MIT License
 
-# 1. Redistributions of source code must retain the above copyright notice, this
-#    list of conditions and the following disclaimer.
+# Copyright (c) 2023 Kshitij Raj Sharma
 
-# 2. Redistributions in binary form must reproduce the above copyright notice,
-#    this list of conditions and the following disclaimer in the documentation
-#    and/or other materials provided with the distribution.
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
 
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
-# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
-# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+# The above copyright notice and this permission notice shall be included in all
+# copies or substantial portions of the Software.
+
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
 
 import ast
 import gzip
@@ -27,7 +27,6 @@ import json
 import os
 import re
 import shutil
-import sys
 import urllib.parse
 from collections import defaultdict
 
@@ -50,149 +49,6 @@ session.mount("https://", retries)
 session.mount("http://", retries)
 
 
-CUSTOM_HEADER = {"user-agent": "oauth_cookie_client.py"}
-
-
-class RaiseError(Exception):
-    def __init__(self, message):
-        self.message = message
-
-
-def report_error(message):
-    sys.stderr.write("{}\n".format(message))
-    raise RaiseError(message)
-
-
-def find_authenticity_token(response):
-    """
-    Search the authenticity_token in the response of the server
-    """
-    pattern = r"name=\"csrf-token\" content=\"([^\"]+)\""
-    m = re.search(pattern, response)
-    if m is None:
-        report_error(
-            "Could not find the authenticity_token in the website to be scraped."
-        )
-    try:
-        return m.group(1)
-    except IndexError:
-        sys.stderr.write(
-            "ERROR: The login form does not contain an authenticity_token.\n"
-        )
-        exit(1)
-
-
-def verify_me_osm(
-    user,
-    password,
-    osm_host="https://www.openstreetmap.org/",
-    consumer_url="https://osm-internal.download.geofabrik.de/get_cookie",
-    format="http",
-):
-
-    username = user
-    if username is None:
-        report_error("The username must not be empty.")
-    if len(password) == 0 or password is None:
-        report_error("The password must not be empty.")
-    if consumer_url is None:
-        report_error("No consumer URL provided")
-
-    # get request token
-    url = consumer_url + "?action=request_token"
-    r = requests.post(url, data={}, headers=CUSTOM_HEADER)
-    if r.status_code != 200:
-        report_error(
-            "POST {}, received HTTP status code {} but expected 200".format(
-                url, r.status_code
-            )
-        )
-    json_response = json.loads(r.text)
-    authorize_url = osm_host + "/oauth/authorize"
-    try:
-        oauth_token = json_response["oauth_token"]
-        oauth_token_secret_encr = json_response["oauth_token_secret_encr"]
-    except KeyError:
-        report_error("oauth_token was not found in the first response by the consumer")
-
-    # get OSM session
-    login_url = osm_host + "/login?cookie_test=true"
-    s = requests.Session()
-    r = s.get(login_url, headers=CUSTOM_HEADER)
-    if r.status_code != 200:
-        report_error("GET {}, received HTTP code {}".format(login_url, r.status_code))
-
-    # login
-    authenticity_token = find_authenticity_token(r.text)
-    login_url = osm_host + "/login"
-    r = s.post(
-        login_url,
-        data={
-            "username": username,
-            "password": password,
-            "referer": "/",
-            "commit": "Login",
-            "authenticity_token": authenticity_token,
-        },
-        allow_redirects=False,
-        headers=CUSTOM_HEADER,
-    )
-    if r.status_code != 302:
-        report_error(
-            "POST {}, received HTTP code {} but expected 302".format(
-                login_url, r.status_code
-            )
-        )
-
-    # authorize
-    authorize_url = "{}/oauth/authorize?oauth_token={}".format(osm_host, oauth_token)
-    r = s.get(authorize_url, headers=CUSTOM_HEADER)
-    if r.status_code != 200:
-        report_error(
-            "GET {}, received HTTP code {} but expected 200".format(
-                authorize_url, r.status_code
-            )
-        )
-    authenticity_token = find_authenticity_token(r.text)
-
-    post_data = {
-        "oauth_token": oauth_token,
-        "oauth_callback": "",
-        "authenticity_token": authenticity_token,
-        "allow_read_prefs": [0, 1],
-        "commit": "Save changes",
-    }
-    authorize_url = "{}/oauth/authorize".format(osm_host)
-    r = s.post(authorize_url, data=post_data, headers=CUSTOM_HEADER)
-    if r.status_code != 200:
-        report_error(
-            "POST {}, received HTTP code {} but expected 200".format(
-                authorize_url, r.status_code
-            )
-        )
-
-    # logout
-    logout_url = "{}/logout".format(osm_host)
-    r = s.get(logout_url, headers=CUSTOM_HEADER)
-    if r.status_code != 200 and r.status_code != 302:
-        report_error(
-            "POST {}, received HTTP code {} but expected 200 or 302".format(logout_url)
-        )
-
-    # get final cookie
-    url = consumer_url + "?action=get_access_token_cookie&format={}".format(format)
-    r = requests.post(
-        url,
-        data={
-            "oauth_token": oauth_token,
-            "oauth_token_secret_encr": oauth_token_secret_encr,
-        },
-        headers=CUSTOM_HEADER,
-    )
-
-    return str(r.text)
-
-
 def create_charts(df, fname):
     ### osm changes block
     # Get the sum of all the create, modify, and delete values
@@ -759,3 +615,48 @@ def update_summary(df1, df2):
     )
     merged_df = merged_df.sort_values("timestamp", ascending=True)
     return merged_df
+
+
+def extract_projects(hashtags):
+    matches = re.findall(r"#hotosm-project-(\d+)", hashtags)
+    return matches
+
+
+def generate_tm_stats(tm_projects, usernames):
+    TM_API_URL = "https://tasking-manager-tm4-production-api.hotosm.org/api/v2/projects"
+    tm_user_stats = {}
+    for project in tm_projects:
+        api_call = f"{TM_API_URL}/{project}/contributions/"
+        response = session.get(api_call)
+        # response.raise_for_status()
+        if response.status_code == 200:
+            data = response.json()
+            for user in data["userContributions"]:
+                if user["username"] in usernames:
+                    user_project_key = f"{user['username']}_{project}"
+                    if user_project_key not in tm_user_stats:
+                        tm_user_stats[user_project_key] = {
+                            "name": user["username"],
+                            "tm_projects": project,
+                            "tm_mapping_level": user["mappingLevel"],
+                            "tasks_mapped": 0,
+                            "tasks_validated": 0,
+                            "tasks_total": 0,
+                        }
+                    tm_user_stats[user_project_key]["tasks_mapped"] += user["mapped"]
+                    tm_user_stats[user_project_key]["tasks_validated"] += user[
+                        "validated"
+                    ]
+                    tm_user_stats[user_project_key]["tasks_total"] += user["total"]
+
+    tm_df = pd.DataFrame(list(tm_user_stats.values()))
+
+    tm_df = tm_df.groupby(["name", "tm_projects"], as_index=False).agg(
+        {
+            "tm_mapping_level": "first",
+            "tasks_mapped": "sum",
+            "tasks_validated": "sum",
+            "tasks_total": "sum",
+        }
+    )
+    return tm_df
